"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.describeReports = exports.report = exports.describe = exports.unsafeFetch = exports.safeFetch = exports.proxyFetch = void 0;
const node_fetch_1 = require("node-fetch");
const types_1 = require("./types");
function send(server, request) {
    const options = {
        method: "POST",
        headers: {
            "Content-Type": "application/json",
            apikey: server.apikey,
        },
        body: JSON.stringify(request),
    };
    return node_fetch_1.default(server.url, options);
}
/**
 * Returns a fetch reponse from specified image proxy with the image data in it
 * @param {Server} server - Server configuration object
 * @param {string} url - Url of the image to fetch
 * @param {ImageProxyDataType} response_type - Data type of response. Either raw bytes or json response
 * @param {boolean} force - Whether or not image should be returned if it is potentially blocked
 * @returns {Promise<FetchResponse | ImageProxyError>} Image proxy reponse or error
 */
function proxyFetch(server, url, response_type, force) {
    return __awaiter(this, void 0, void 0, function* () {
        const fetchRequest = {
            jsonrpc: server.version,
            method: types_1.ImageProxyMethod.Fetch,
            params: {
                response_type,
                url,
                force,
            },
        };
        const response = (yield send(server, fetchRequest)).text().then((text) => {
            try {
                return JSON.parse(text);
            }
            catch (e) {
                return text;
            }
        });
        return response;
    });
}
exports.proxyFetch = proxyFetch;
/**
 * Returns a moderated fetch reponse from specified image proxy with the image data in it, blocking unsafe images
 * @param {Server} server - Server configuration object
 * @param {string} url - Url of the image to fetch
 * @param {ImageProxyDataType} response_type - Data type of response. Either raw bytes or json response
 * @returns {Promise<FetchResponse | ImageProxyError>} Image proxy reponse or error
 */
function safeFetch(server, url, response_format) {
    return __awaiter(this, void 0, void 0, function* () {
        return proxyFetch(server, url, response_format, false);
    });
}
exports.safeFetch = safeFetch;
/**
 * Returns an unmoderated fetch reponse from specified image proxy with the image data in it, allowing all images
 * @param {Server} server - Server configuration object
 * @param {string} url - Url of the image to fetch
 * @param {ImageProxyDataType} response_type - Data type of response. Either raw bytes or json response
 * @returns {Promise<FetchResponse | ImageProxyError>} Image proxy reponse or error
 */
function unsafeFetch(server, url, response_format) {
    return __awaiter(this, void 0, void 0, function* () {
        return proxyFetch(server, url, response_format, true);
    });
}
exports.unsafeFetch = unsafeFetch;
/**
 * Describes the moderation status of provided urls
 * @param {Server} server - Server configuration object
 * @param {string} urls - List of urls of the images to describe
 * @returns {Promise<DescribeResponse | ImageProxyError>} Describe reponse or error
 */
function describe(server, urls) {
    return __awaiter(this, void 0, void 0, function* () {
        const describeRequest = {
            jsonrpc: server.version,
            method: types_1.ImageProxyMethod.Describe,
            params: { urls },
        };
        return send(server, describeRequest).then((response) => response.json());
    });
}
exports.describe = describe;
/**
 * Report a url that was deemed safe but should be blocked
 * @param {Server} server - Server configuration object
 * @param {string} urls - List of urls of the images to report
 * @param {ModerationLabel[]} categories - List of moderation labels to report the image as
 * @returns {Promise<ReportResponse | ImageProxyError>} Report reponse or error
 */
function report(server, url, categories) {
    return __awaiter(this, void 0, void 0, function* () {
        const reportRequest = {
            jsonrpc: server.version,
            method: types_1.ImageProxyMethod.Report,
            params: { url, categories },
        };
        return send(server, reportRequest).then((response) => response.json());
    });
}
exports.report = report;
/**
 * Describe all reports submitted to image proxy
 * @param {Server} server - Server configuration object
 * @returns {Promise<DescribeReportResponse | ImageProxyError>} Describe report response or error
 */
function describeReports(server) {
    return __awaiter(this, void 0, void 0, function* () {
        const describeReportsRequest = {
            jsonrpc: server.version,
            method: types_1.ImageProxyMethod.DescribeReports,
        };
        return send(server, describeReportsRequest).then((response) => response.json());
    });
}
exports.describeReports = describeReports;
